import QtQuick 2.0
import Sailfish.Silica 1.0

import dev.neko.spoon 1.0

Page {
    id: chatPage

    // The effective value will be restricted by ApplicationWindow.allowedOrientations
    allowedOrientations: Orientation.All

    property alias room: chatView.model

    Column {
        anchors.fill: parent

        PageHeader {
            id: header
            title: room.roomName()

            Connections {
                target: room
                onRoomNameChanged: header.title = name
            }
        }

        SilicaListView {
            id: chatView

            height: parent.height - bot.height - header.height
            contentHeight: height
            cacheBuffer: 1000
            clip: true

            ScrollDecorator { flickable: chatView }
            boundsBehavior: Flickable.DragOverBounds

            anchors.left: parent.left
            anchors.right: parent.right

            anchors.leftMargin: Theme.horizontalPageMargin
            anchors.rightMargin: Theme.horizontalPageMargin

            property bool atEnd: false
            ListView.onAdd: {
                console.log("atYEnd::" + atYEnd)
                atEnd = atYEnd
            }
            onCountChanged: {
                console.log("atEnd::" + atEnd)
                if (atEnd)
                scrollToBottom()
            }

            onMovementEnded: {
                atEnd = false
                console.log("movement ended")
                if (chatView.atYBeginning) {
                    console.log("load older messages")
                    room.fetchOlderMessages()
                }
                if (chatView.atYEnd) {
                    console.log("at end")
                    atEnd = true
                }

                var lastIndex = maxIndex
                for(var child in chatView.contentItem.children) {
                    if (chatView.contentItem.children[child].isFullyVisible && chatView.contentItem.children[child].index > maxIndex)
                        lastIndex = chatView.contentItem.children[child].index;
                    }
                    if (lastIndex > maxIndex) {
                        maxIndex = lastIndex
                    }
            }

            property int maxIndex: -1
            onMaxIndexChanged: {
                console.log("max index: " + maxIndex)
                room.markRead(maxIndex)
            }

            delegate: ListItem {
                id: messageItem
                contentHeight: loader.item.height

                property int yoff: Math.round(messageItem.y - chatView.contentY - 1)
                property bool isFullyVisible: yoff + messageItem.height < chatView.height
                property int index: model.index

                Loader {
                    id: loader
                    source: switch(model.Type) {
                        case EventType.Aliases: return "delegates/Aliases.qml"
                        case EventType.Avatar: return "delegates/Avatar.qml"
                        case EventType.CanonicalAlias: return "delegates/CanonicalAlias.qml"
                        case EventType.Create: return "delegates/Create.qml"
                        case EventType.GuestAccess: return "delegates/GuestAccess.qml"
                        case EventType.HistoryVisibility: return "delegates/HistoryVisibility.qml"
                        case EventType.JoinRules: return "delegates/JoinRules.qml"
                        case EventType.Member: return "delegates/Member.qml"
                        case EventType.Name: return "delegates/Name.qml"
                        case EventType.PowerLevels: return "delegates/PowerLevels.qml"
                        case EventType.Topic: return "delegates/Topic.qml"
                        case EventType.NoticeMessage: return "delegates/NoticeMessage.qml"
                        case EventType.TextMessage: return "delegates/TextMessage.qml"
                        case EventType.ImageMessage: return "delegates/ImageMessage.qml"
                        case EventType.VideoMessage: return "delegates/VideoMessage.qml"
                        default: return "delegates/placeholder.qml"
                    }
                    property variant modelData: model
                }
            }

            section.property: "UserId"
            section.criteria: ViewSection.FullString
            section.delegate: Text {
                text: room.userIdToUserName(section)
                color: room.userColor(section, "#00000000")
            }

        }

        Row {
            id: bot

            anchors.left: parent.left
            anchors.right: parent.right
            height: messageText.height

            TextArea {
                id: messageText

                anchors.bottom: parent.bottom

                placeholderText: qsTr("Enter your message")
                wrapMode: Text.Wrap
                width: parent.width - sendButton.width
            }

            IconButton {
                id: sendButton

                anchors.top: parent.top

                icon.source: "image://theme/icon-m-enter"
                onClicked: {
                    if (messageText.text != "") {
                        room.sendTextMessage(messageText.text)
                    }
                    messageText.text = ""
                }
            }
        }
    }
}
