import QtQuick 2.0
import Sailfish.Silica 1.0

import QtMultimedia 5.6
import Sailfish.Media 1.0

Rectangle {
    id: container
    height: col.height

    Column {
        id: col

        Item {
            id: videoContainer
            height: chatView.width
            width: chatView.width

            Image {
                id: thumb
                fillMode: Image.PreserveAspectFit
                width: chatView.width
                height: thumb.status == Image.Error ? chatView.width : undefined
                sourceSize.width: chatView.width
                sourceSize.height: chatView.width
                source: modelData.Url.replace("mxc://", "image://MXCImage/")
                asynchronous: true
                visible: player.download == undefined

                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        if (player.download == undefined) {
                            player.download = mxcDownloader.download(modelData.Url)
                        }
                    }
                }

                Rectangle {
                    color: '#00000050'
                    anchors.fill: parent
                    visible: thumb.status == Image.Error
                    height: chatView.width
                }

                Image {
                    id: downloadButton
                    anchors.centerIn: thumb
                    visible: thumb.status == Image.Ready || thumb.status == Image.Error
                    //source: "image://theme/icon-m-play"
                    source: "image://theme/icon-m-cloud-download"
                }

                BusyIndicator {
                    id: busy
                    size: BusyIndicatorSize.Medium
                    anchors.centerIn: thumb
                    running: thumb.status == Image.Loading
                }
            }

            Rectangle {
                anchors.fill: parent
                visible: player.download != undefined
                color: "black"

                MediaPlayer {
                    property var download: mxcDownloader.cached(modelData.Url) ? mxcDownloader.download(modelData.Url) : undefined 
                    id: player
                    source: download.location
                    autoLoad: true
                }

                VideoOutput {
                    id: videoOutput
                    source: player
                    anchors.fill: parent
                    visible: player.download != undefined

                    MouseArea {
                        anchors.fill: parent
                        onClicked: overlay.open = !overlay.open
                        //{
                        //    if(overlay.open)
                        //    overlay.open = false
                        //    else
                        //    overlay.open = true
                        //}
                    }
                }

                MediaPlayerControlsPanel {
                    id: overlay
                    anchors.left: parent.left
                    anchors.right: parent.right
                    anchors.bottom: parent.bottom
                    z:100
                    duration: player.duration / 1000
                    position: player.position / 1000
                    playing: player.playbackState == MediaPlayer.PlayingState
                    showMenu: false
                    open: true
                    onSliderReleased: {
                        player.seek(value * 1000)
                    }
                    onPreviousClicked: player.seek(0)
                    onPlayPauseClicked:  {
                        if (player.playbackState == MediaPlayer.PlayingState) {
                            player.pause()
                        }
                        else {
                            player.play()
                            overlay.open = false
                        }
                    }

                    //Component.onCompleted: overlay.showControls()
                }
            }
        }


        Text {
            id: body
            text: modelData.FormattedBody
            color: Theme.secondaryColor
            wrapMode: Text.Wrap
            width: chatView.width
            font.pixelSize: Theme.fontSizeTiny
        }
        Text {
            id: ts
            anchors.right: col.right
            text: modelData.Timestamp.toLocaleTimeString()
            color: Theme.highlightColor
            font.pixelSize: Theme.fontSizeTiny
            horizontalAlignment: Text.AlignRight
        }
    }
}
